/*
 * $Id: libqg.h,v 1.38 2003/10/06 23:33:55 ldemailly Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/**
 * libqg.h
 *
 * This headerfile contains all definitions of macros, variables and
 * functions which have to be accessible by programs using libqg to
 * connect to QualysGuard.
 */

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/wait.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <dirent.h>
#include <curl/curl.h>
#include <curl/types.h>
#include <string.h>
#include <strings.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>

typedef struct _qgresponse {
    CURLcode res;
    char *data;
    size_t size;
    xmlDocPtr xml;
    int  httpCode;
} qgresponse_t;

typedef struct _qghandle {
    CURL *curl;
} qghandle_t;

typedef struct ipList {
    in_addr_t ip;
    long count;
    struct ipList *next;
} ts_ipList, *tsp_ipList;

// Max number of IPs to enumerate in an ipList
#define MAX_IPS 10000

// Output messages
typedef enum {
    DBG,
    ERR,
    SYS
} QG_mtype;


// Buffer length for 'reasonable' length data/debug/diagnostic/snprintf/...
#define STR_BUF_LEN 1024

#define QGSERVER "https://qualysguard.qualys.com"

// No result at search
#define LIBQG_NOT_FOUND "NOT_FOUND"
#define LIBQG_NFC_CONST 9

// Init Function
qghandle_t *qg_init(char *username, char *password, char *baseUrl, int debug);

// Scan Functions
qgresponse_t *qg_scan(qghandle_t * qgh, char *addrs, int save, char *isname);
qgresponse_t *qg_scan_running_list(qghandle_t * qgh);
qgresponse_t *qg_scan_cancel(qghandle_t * qgh, char *ref);
qgresponse_t *qg_scan_report_list(qghandle_t * qgh, int last, char *addrs);
qgresponse_t *qg_scan_report(qghandle_t * qgh, char *ref, char *addrs);
qgresponse_t *qg_scan_report_delete(qghandle_t * qgh, char *ref);

// Map Functions
qgresponse_t *qg_map(qghandle_t * qgh, char *domain, int save, char *sname);
qgresponse_t *qg_map_report_list(qghandle_t * qgh, int last, char *domain);
qgresponse_t *qg_map_report(qghandle_t * qgh, char *ref);

// Preferences
qgresponse_t *qg_scan_options(qghandle_t * qgh, int scandeadhosts,
                              int bandwidth, int loadbalancer, char *ports);
qgresponse_t *qg_group_list(qghandle_t *qgh);
tsp_ipList qg_ips_from_group(qghandle_t *qgh, char *ipgroup);
void freeIpList(tsp_ipList ipList);

// Misc
qgresponse_t *qg_cve_list(qghandle_t *qgh);
xmlNodePtr qg_xmlroot(xmlDocPtr doc, char *start);

// Basic XML Parsing routines
xmlNodePtr qg_xml_find_ip(xmlDocPtr doc, char *ipstr);

// XML Utility Functions
char *qg_xml_find_os(xmlNodePtr iproot);
char *qg_xml_get_report_ref(xmlDocPtr doc);
xmlDocPtr qg_custom_report(xmlDocPtr report, char *ip, char *cachedir);

// Misc/Util functions
void qg_free_response(qgresponse_t * qgr);
char *strqg_error(char *message);
char *qg_strappend(char *base, char *addstr);

// extern variables for overriding default values
// for debugging, certificate and base url after qg_init is called
// (configuration (Makefile) & qg_init arguments are the preferred
//  and supported way to change those)
extern int qg_debug;
extern char *base_url;
extern char *qg_err_message;
extern char *program;
// QG API Version information
extern int qg_api_major;
extern int qg_api_minor;
