/*
 * $Id: libqgInt.h,v 1.20 2003/09/24 02:06:24 wkoeppl Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/**
 * libqgInt.h
 *
 * This file contains definitions of macros, variables and functions
 * only used by libqg, which should not be used and changed by 
 * functions outside libqg.
 */

#include "libqg.h"
#include <stdarg.h>

//MSP directory
#define MSP_DIR "msp"

// QualysGuard URLs
// QWEB before 2.16, about.php does not exist (= API 1.0)
#if (API_VERSION_MINOR == 0)
#  define PING_URL "scan_running_list.php"
#else
// 1.1 or later:
#  define PING_URL "about.php"
#endif
#define S_URL "scan.php?"
#define SOPTIONS_URL "scan_options.php?"
#define SLIST_URL "scan_running_list.php"
#define SCANCEL_URL "scan_cancel.php?"
#define SREPORTLIST_URL "scan_report_list.php?"
#define SREPORT_URL "scan_report.php?"
#define SREPORTDEL_URL "scan_report_delete.php?"
#define M_URL "map.php?"
#define MREPORTLIST_URL "map_report_list.php?"
#define MREPORT_URL "map_report.php?"

// HTTP check defines
#define HTTP_ERRORS_START 300  // 300 or greater = errors
#define HTTP_AUTH_FAIL  401

// Scan option defines
#define	QGOPTIONS_BW_LOW			1
#define	QGOPTIONS_BW_MEDIUM			2
#define	QGOPTIONS_BW_HIGH			3
#define	QGOPTIONS_BW_MAXIMUM		4

// Error messages
#define LIBQG_ERR_MALLOC "Memory allocation failed"
#define LIBQG_ERR_REALLOC "Memory reallocation failed"
#define LIBQG_ERR_STRDUP "strdup failed"
#define LIBQG_ERR_CURL "Error code %d for \"%s\":\n%s"
#define LIBQG_ERR_MISPAR "missing parameter(s): %s"
#define LIBQG_ERR_SSL "Verification of SSL-Certificate of Server failed."

// Debug messages
#define LIBQG_FETCH_URL "libqg fetching url \"%s\""

// Function names for error messages
#define LIBQG_INIT "qg_init"
#define LIGQG_IPS_FROM_GROUP "qg_ips_from_group"
#define LIBQG_MAP "qg_map"
#define LIBQG_MAP_REP "qg_map_report"
#define LIBQG_MAP_REP_LIS "qg_map_report_list"
#define LIBQG_SCAN "qg_scan"
#define LIBQG_SCAN_CAN "qg_scan_cancel"
#define LIBQG_SCAN_OPT "qg_scan_options"
#define LIBQG_SCAN_REP "qg_scan_report"
#define LIBQG_SCAN_DEL "qg_scan_report_delete"
#define LIBQG_SCAN_REP_LIS "qg_scan_report_list"
#define LIBQG_SCAN_RUN_LIS "qg_scan_running_list"
#define LIBQG_RECV "qg_recv"
#define LIBQG_FETCH "qg_fetch"
#define LIBQG_STRAP "qg_strappend"
#define LIBQG_VERI_LOG "qg_verify_login"
#define LIBQG_XMLU_FETREP "qg_xml_fetch_report"
#define LIBQG_XMLU_VULNCV "qg_xml_vuln_cve"

// Function arguments
#define ARG_DOMAIN "domain="
#define ARG_SAVEREP_Y "&save_report=yes"
#define ARG_SAVEREP_N "&save_report=no"
#define ARG_REFER "ref="
#define ARG_ISCAN_NAME "&iscanner_name="
#define ARG_LAST_Y "last=yes"
#define ARG_LAST_N "last=no"
#define ARG_TARG "&target="

//XML-comparison
#define XML_SERV    "SERVICES"
#define XML_RESU    "RESULT"
#define XML_VALU    "value"

#define XMLPATH_VULN "/SCAN/IP[@value=\"%s\"]/VULNS/CAT/VULN[@number]"
#define XMLPATH_THREAT "/SCAN/IP[@value=\"%s\"]/PRACTICES/CAT/PRACTICE[@number]"
#define XMLPATH_TCP "/SCAN/IP[@value=\"%s\"]/SERVICES/CAT/SERVICE[@value=\"scan_tcp\"]/RESULT"
#define XMLPATH_UDP "/SCAN/IP[@value=\"%s\"]/SERVICES/CAT/SERVICE[@value=\"scan_udp\"]/RESULT"
#define XMLPATH_WEBSERV "/SCAN/IP[@value=\"%s\"]/SERVICES/CAT[@value=\"http\"][@port=\"%s\"]/SERVICE[@value=\"httpd_version\"]/RESULT"

//CVE checking
#define CVE_PREFIX_LEN 4

#define SHORT_BUF 10

typedef enum {
    QG_TCP,
    QG_UDP,
    QG_VULN,
    QG_THREAT
} QG_custrep;

typedef struct custrep_t {
    xmlXPathContextPtr qgcontext;
    xmlXPathContextPtr qidtoids;
    char *ip;
    double vuln;
    double vulnmap;
    double threat;
    double threatmap;
} custrep_t;

// Verify Login
int qg_verify_login(qghandle_t *qgh);

// XML utilities
void qg_xml_custom_procvuln(custrep_t *crep,  xmlNodePtr report, QG_custrep type);
void qg_xml_custom_port(xmlXPathContextPtr qgcontext, xmlNodePtr parent, QG_custrep protocol, char *ip);
void qg_xml_get_webservname(xmlXPathContextPtr qgcontext, xmlNodePtr port, char *ip);
int findInFirstColumn(char* wholeString,char* searchString);
int findColEnd(char* string);

// Basic XML Parsing routines
xmlNodePtr qg_xml_find_child(xmlNodePtr parent, char *name);
xmlNodePtr qg_xml_find_prop(xmlNodePtr parent, char *name, char *value);
void qg_xml_print_children(xmlNodePtr parent);
u_int qg_xml_check_atlogin(xmlDocPtr doc, char *url);

// Misc/Util functions
size_t qg_recv(void *buffer, size_t size, size_t nmemb, void *userp);
qgresponse_t *qg_fetch_raw(qghandle_t * qgh, char *url);
qgresponse_t *qg_fetch(qghandle_t * qgh, char *url);
void qg_error(int app, char *usedfunc, const char *error,...)__attribute__((format(printf,3,4)));
void libqg_message(int type, const char *message,...)__attribute__((format(printf,2,3)));

#define LIBQG_DEBUG 0

#if (LIBCURL_VERSION_NUM < 0x070a00)
#  error "Your version of the curl library is too old, please use > 7.10.0"
#endif

#if (LIBXML_VERSION < 20400)
#  error "Your version of the libxml2 library is too old, please use > 2.4"
#endif
