/*
 * $Id: qg_group_list.c,v 1.6 2003/10/06 19:24:43 wkoeppl Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "libqgInt.h"

/**
 * qg_group_list calls the QualysGuard group_list api.
 */
qgresponse_t *qg_group_list(qghandle_t *qgh)
{
    qgresponse_t *qgr = NULL;
    char url[STR_BUF_LEN];

    if (!qgh) {
        qg_error(FALSE, "qg_group_list", LIBQG_ERR_MISPAR, "qghandle");
        return (NULL);
    }

    snprintf(url, STR_BUF_LEN, "%s/%s/%s",
             base_url, MSP_DIR, "group_list.php");
    qgr = qg_fetch(qgh, url);
    return (qgr);
}

/**
 * Insert a new entry for that IP in an existing (possibly NULL)
 * ipList. The new entry becomes the new head of the list.
 */
tsp_ipList insertNode(in_addr_t ip, tsp_ipList prevHead) {
    // possible TODO: check for invalid ip
    tsp_ipList node = calloc(1, sizeof(ts_ipList));
    node->ip = ip;
    node->count = (prevHead != NULL) ? prevHead->count + 1 : 1;
    node->next = prevHead;
    return node;
}
tsp_ipList ipRangeEnumFromIPs(in_addr_t ip1, in_addr_t ip2, tsp_ipList list) {
    // possible TODO: range checking
    long ip1l= ntohl(ip1), ip2l = ntohl(ip2);
    for (; ip1l <= ip2l; ip1l++) {
        list = insertNode(htonl(ip1l), list);
        if (list->count > MAX_IPS) {
            return list;
        }
    }
    return list;
}

tsp_ipList ipRangeEnumFromStr(char *ipRangeStr, tsp_ipList last) {
    char *cur;
    in_addr_t ip1, ip2;
    libqg_message(DBG, "Called ipRangeEnumFromStr for \"%s\"", ipRangeStr);
    for (cur = ipRangeStr; *cur != 0; cur++) {
        if (*cur == '-') {
            // It's a range
            *cur = 0;
            ip1 = inet_addr(ipRangeStr);
            ip2 = inet_addr(cur+1);
            return ipRangeEnumFromIPs(ip1, ip2, last);
        }
    }
    return insertNode(inet_addr(ipRangeStr), last);
}

/**
 * Enumerates an xml node set of IP ranges into an ipList linked list.
 */
tsp_ipList ipRanges2ipList(xmlXPathObjectPtr in) {
    tsp_ipList result = NULL;
    int i;
    for (i=0; i < in->nodesetval->nodeNr; i++) {
        result = ipRangeEnumFromStr(in->nodesetval->nodeTab[i]->children->content, result);
        if (result->count > MAX_IPS) {
            qg_error(FALSE, LIGQG_IPS_FROM_GROUP, "Too many IPs (> %d) specified", MAX_IPS);
            return NULL;
        }
    }
    return result;
}


/**
 * qg_ips_from_group returns the xpatch objects with the 
 * ips in the NodeSet or NULL if no IPs where found or an error
 * occured
 */
tsp_ipList qg_ips_from_group(qghandle_t *qgh, char *ipgroup)
{
    tsp_ipList res = NULL;
    qgresponse_t *qgGroups = NULL;
    xmlXPathContextPtr groupContext = NULL;
    xmlXPathObjectPtr  groupXP = NULL;
    char groupXPstr[STR_BUF_LEN];
    qgGroups = qg_group_list(qgh);
    if (qgGroups == NULL) {
        // error already set by callees
        qg_error(TRUE, LIGQG_IPS_FROM_GROUP, "for group \"%s\"", ipgroup);
        return NULL;
    }
    groupContext = xmlXPathNewContext(qgGroups->xml);
    snprintf(groupXPstr, STR_BUF_LEN, 
             "/GROUP_LIST/GROUP[NAME=\"%s\"]/SCANIPS/IP", 
             ipgroup);
    groupXP = xmlXPathEvalExpression(groupXPstr, groupContext);
    if (groupXP==NULL) {
        qg_error(FALSE, LIGQG_IPS_FROM_GROUP, "Error compiling xpath \"%s\"", 
                 groupXPstr);
        xmlXPathFreeContext(groupContext);
        qg_free_response(qgGroups);
        return NULL;
    }
    if (xmlXPathNodeSetIsEmpty(groupXP->nodesetval)) {
        qg_error(FALSE, LIGQG_IPS_FROM_GROUP, "Group \"%s\" not found (xp='%s')", 
                     ipgroup, groupXPstr);
    } else {
        res = ipRanges2ipList(groupXP);
    }
    xmlXPathFreeObject(groupXP);
    xmlXPathFreeContext(groupContext);
    qg_free_response(qgGroups);
    return res;
}

/**
 * Free all the entry of a list of IPs.
 */
void freeIpList(tsp_ipList ipList){
    tsp_ipList ipListChild;
    while (ipList != NULL) {
        ipListChild = ipList->next;
        free(ipList);
        ipList = ipListChild;
    }
}
