/*
 * $Id: qg_init.c,v 1.17 2003/08/19 02:20:57 mkrieger Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "libqgInt.h"
#include <sys/utsname.h>
#include <errno.h>

char *qg_cafile = NULL;
int qg_debug = 0;
char *base_url = NULL;
char *qg_err_message = NULL;
char *program = NULL;

/**
 * qg_init sets the global curl-options and checks for the existence of 
 * username and password. It calls qg_verify_login to verify, that a 
 * connection to QualysGuard can be successfully established.
 * returns a qghandle on success and NULL on error
 */
qghandle_t *qg_init(char *username, char *password, char *url, int debug)
{
    qghandle_t *qgh = NULL;
    char buf[STR_BUF_LEN];
    char userbuf[STR_BUF_LEN];
    char *useragent = NULL;
    struct utsname pc;
    int p;

    qg_debug = debug;
    if (program == NULL) {
        program = "NOT SPECIFIED";
    }

    //check for url
    if (url != NULL) {
        base_url = url;
    } else {
        base_url = QGSERVER;
    }

    if ((p = uname(&pc)) == -1) {
        char *err = strerror(errno);
        libqg_message(0, "error in uname: %s", err);
        snprintf(userbuf, STR_BUF_LEN, "%s / libqg %d.%d (uname-error %s)",
                 program, API_VERSION_MAJOR, API_VERSION_MINOR,
                 err);
    } else {
        snprintf(userbuf, STR_BUF_LEN, "%s / libqg %d.%d (%s %s %s)",
                 program, API_VERSION_MAJOR, API_VERSION_MINOR, 
                 pc.sysname, pc.release, pc.machine);
    }
    useragent = strdup(userbuf);

    // Malloc qghandle
    if ((qgh = (qghandle_t *) malloc(sizeof(qghandle_t))) == NULL) {
        qg_error(0, LIBQG_INIT, LIBQG_ERR_MALLOC);
        return (NULL);
    }

    // check ret code
    if(curl_global_init(CURL_GLOBAL_ALL) != 0){
        qg_error(0, LIBQG_INIT, "Your libcurl does not work properly. Please check the installation of libcurl");
        return (NULL);
    }

    // Init CURL
    if ((qgh->curl = curl_easy_init()) == NULL) {
        free(qgh);
        qg_error(0, LIBQG_INIT, "Libcurl was not correctly initialized");
        return (NULL);
    }

    snprintf(buf, STR_BUF_LEN, "%s:%s", username, password);
    //Setting the CURL-Options
    //Login
    curl_easy_setopt(qgh->curl, CURLOPT_USERPWD, strdup(buf));
    curl_easy_setopt(qgh->curl, CURLOPT_USERAGENT, useragent);
    libqg_message(0, "User-Agent: set to \"%s\"", useragent);

    // SSL? - if yes, verify server and certificate
    if (strncasecmp(base_url, "https://", 8) == 0) {
        curl_easy_setopt(qgh->curl, CURLOPT_SSL_VERIFYHOST, 2);
        curl_easy_setopt(qgh->curl, CURLOPT_SSL_VERIFYPEER, 2);
    }

    libqg_message(0, "libqg init + checking connection to server");
    
    // check login
    if (qg_verify_login(qgh)) {
        curl_global_cleanup();
        return NULL;
    }
    
    libqg_message(0, "libqg init complete");

    return (qgh);
}
