/*
 * $Id: qg_scan.c,v 1.12 2003/08/19 02:20:57 mkrieger Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "libqgInt.h"

/**
 * qg_scan connects to QualysGuard with qg_fetch and orders to start a
 * scan off the IP-address 'addrs'. The report is saved if 'save' is set
 * to 1. If you use an Intranet Scanner 'sname' will be required to start
 * the scan.
 * returns NULL on error
 */
qgresponse_t *qg_scan(qghandle_t *qgh, char *addrs, int save, char *sname)
{
    char baseurl[STR_BUF_LEN];
    char *url = NULL;
    qgresponse_t *qgr = NULL;

    if (!addrs || !qgh) {
        qg_error(0, LIBQG_SCAN, LIBQG_ERR_MISPAR, "qghandle / ip-address");
        return (NULL);
    }

    snprintf(baseurl, STR_BUF_LEN, "%s/%s/%s", base_url, MSP_DIR, S_URL);

    // Copy base url
    if ((url = (char *) strdup(baseurl)) == NULL) {
        qg_error(0, LIBQG_SCAN, LIBQG_ERR_STRDUP);
        return (NULL);
    }
    // Addresses
    url = qg_strappend(url, "ip=");
    url = qg_strappend(url, addrs);

    // Save on Server?
    if (save) {
        url = qg_strappend(url, ARG_SAVEREP_Y);
    } else {
        url = qg_strappend(url, ARG_SAVEREP_N);
    }

    // Iscanner specified?
    if (sname) {
        url = qg_strappend(url, ARG_ISCAN_NAME);
        url = qg_strappend(url, sname);
    }
    // fetch page
    qgr = qg_fetch(qgh, url);
    free(url);
    return (qgr);
}
