/*
 * $Id: qg_scan_options.c,v 1.14 2003/08/19 02:20:57 mkrieger Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "libqgInt.h"

/**
 * qg_scan_options connects to QualysGuard with qg_fetch and sets the
 * scan-options which can be specified with the arguments 'scandeadhosts',
 * 'bandwith', 'loadbalancer', 'ports'
 * returns NULL on error
 */
qgresponse_t *qg_scan_options(qghandle_t *qgh, int scandeadhosts,
                              int bandwidth, int loadbalancer, char *ports)
{
    char baseurl[STR_BUF_LEN];
    char *url = NULL;
    qgresponse_t *qgr = NULL;

    if (!qgh || !bandwidth) {
        qg_error(0, LIBQG_SCAN_OPT, LIBQG_ERR_MISPAR, "qghandle / bandwidth");
        return (NULL);
    }

    snprintf(baseurl, STR_BUF_LEN, "%s/%s/%s", 
             base_url, MSP_DIR, SOPTIONS_URL);
    // Copy base url
    if ((url = (char *) strdup(baseurl)) == NULL) {
        qg_error(0, LIBQG_SCAN_OPT, LIBQG_ERR_STRDUP);
        return (NULL);
    }
    // Scan Dead hosts?
    if (scandeadhosts) {
        url = qg_strappend(url, "scandeadhosts=yes");
    } else {
        url = qg_strappend(url, "scandeadhosts=no");
    }

    // Which bandwidth option?
    switch (bandwidth) {
        case QGOPTIONS_BW_LOW:
            url = qg_strappend(url, "&bandwidth=low");
            break;

        case QGOPTIONS_BW_MEDIUM:
            url = qg_strappend(url, "&bandwidth=medium");
            break;

        case QGOPTIONS_BW_HIGH:
            url = qg_strappend(url, "&bandwidth=high");
            break;

        case QGOPTIONS_BW_MAXIMUM:
            url = qg_strappend(url, "&bandwidth=maximum");
            break;

        default:
            free(url);
            qg_error(0, LIBQG_SCAN_OPT, "The bandwith option was not correctly specified.");
            return (NULL);
            break;
    }

    // Load Balancer
    if (loadbalancer) {
        url = qg_strappend(url, "&loadbalancer=yes");
    } else {
        url = qg_strappend(url, "&loadbalancer=no");
    }

    // Ports
    if (!ports) {
        url = qg_strappend(url, "&ports=default");
    } else {
        url = qg_strappend(url, "&ports=");
        url = qg_strappend(url, ports);
    }

    // fetch page
    qgr = qg_fetch(qgh, url);
    free(url);
    return (qgr);
}
