/*
 * $Id: qg_scan_report_list.c,v 1.13 2003/08/19 02:20:57 mkrieger Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "libqgInt.h"

/**
 * qg_scan_report_list connects with qg_fetch to QualysGuard and 
 * returns a XML-document containing a list of reports
 * 'last' is the flag, if the XML-document should contain only the
 * last report, 'addrs' specifies a IP-address for which all saved
 * report should be listed.
 * qg_scan_report_list returns NULL on error
 */
qgresponse_t *qg_scan_report_list(qghandle_t *qgh, int last, char *addrs)
{
    char baseurl[STR_BUF_LEN];
    char *url = NULL;
    qgresponse_t *qgr = NULL;

    if (!qgh) {
        qg_error(0, LIBQG_SCAN_REP_LIS, LIBQG_ERR_MISPAR, "qghandle");
        return (NULL);
    }

    snprintf(baseurl, STR_BUF_LEN, "%s/%s/%s", 
             base_url, MSP_DIR, SREPORTLIST_URL);
    // Copy base url
    if ((url = (char *) strdup(baseurl)) == NULL) {
        qg_error(0, LIBQG_SCAN_REP_LIS, LIBQG_ERR_STRDUP);
        return (NULL);
    }
 
    // View last?
    if (last) {
        url = qg_strappend(url, ARG_LAST_Y);
    } else {
        url = qg_strappend(url, ARG_LAST_N);
    }

    // Addresses
    if (addrs) {
        url = qg_strappend(url, ARG_TARG);
        url = qg_strappend(url, addrs);
    }
    // fetch page

    qgr = qg_fetch(qgh, url);
    free(url);
    return (qgr);
}
