/*
 * $Id: qg_verify.c,v 1.24 2003/10/07 02:00:13 ldemailly Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "libqgInt.h"

int qg_api_major = 0;
int qg_api_minor = 0;

/**
 * qg_verify_login connects to QualysGuard by calling qg_fetch() and 
 * checks with the returned HTTP-code if the connection was established.
 * It also checks the API-version of QualysGuard. qg_verify_login returns
 * FALSE if the login was successful and TRUE if an error occurred.
 */
int qg_verify_login(qghandle_t *qgh)
{
    qgresponse_t *qgr = NULL;
    char url[STR_BUF_LEN];
    xmlNodePtr cur;

    qg_api_major = qg_api_minor = 0;

    if (!qgh) {
        qg_error(0, LIBQG_VERI_LOG, LIBQG_ERR_MISPAR, "qgh");
        return TRUE;
    }

    // Connect and get scan_running_list.php/about.php 
    snprintf(url, STR_BUF_LEN, "%s/%s/%s", base_url, MSP_DIR, PING_URL);

    if((qgr = qg_fetch(qgh, url)) == NULL){
        qg_error(1, LIBQG_VERI_LOG, "Error in reply from Server.");
        return TRUE;
    }

    if ((cur = xmlDocGetRootElement(qgr->xml)) == NULL) {
        qg_error(0, LIBQG_VERI_LOG, "No root element found in %s.\n", url);
        qg_free_response(qgr);
        return TRUE;
    }

    // Check the response:
    for (cur = cur->children; cur != NULL; cur = cur->next) {
        // Version compatibility [change when/if 2.0 is supported]
#if (API_VERSION_MINOR > 0)
        if (strcmp("API-VERSION", cur->name) == 0) {
            char *major_s = xmlGetProp(cur, "MAJOR");
            char *minor_s = xmlGetProp(cur, "MINOR");
            qg_api_major = major_s ? atoi(major_s) : -1;
            qg_api_minor = minor_s ? atoi(minor_s) : -1;
            libqg_message(0, "libqg found remote version=\"%s.%s\"", 
                             major_s,
                             minor_s);
            // check for exactly 1.x ; x >= 1 :
            if (qg_api_major != API_VERSION_MAJOR 
                || qg_api_minor < API_VERSION_MINOR) {
                qg_error(0, LIBQG_VERI_LOG, "Incompatible API version %d.%d, expecting %d.x with x >= %d\n",
                         qg_api_major, qg_api_minor, 
                         API_VERSION_MAJOR, API_VERSION_MINOR);
                free(major_s);
                free(minor_s);
                qg_free_response(qgr);
                return TRUE;
            }
            free(major_s);
            free(minor_s);
            qg_free_response(qgr);
            return FALSE; // Correct Version!
        }
#endif
    }

#if (API_VERSION_MINOR > 0)
    qg_error(0, LIBQG_VERI_LOG, "API Version information was not found in returned document\n%s\n", 
           qgr->data);
    qg_free_response(qgr);
    return TRUE;
#else
    qg_free_response(qgr);
    return FALSE; // Should be ok (but can't check version)
#endif
}
