/*
 * $Id: qg_xmlparse.c,v 1.13 2003/08/11 23:34:26 mkrieger Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "libqgInt.h"

/**
 * qg_xml_find_ip searches in the XML-document specified by 'doc'
 * for the matching IP-string and returns the node where it was
 * found. On failure NULL is returned.
 */
xmlNodePtr qg_xml_find_ip(xmlDocPtr doc, char *ipstr)
{
    xmlNodePtr cur = NULL;
    char *curval = NULL;

    // Sanity check
    if (!doc || !ipstr) {
        return (NULL);
    }

    // Get root item
    if ((cur = xmlDocGetRootElement(doc)) == NULL) {
        return (NULL);
    }

    // Search for the item that matches the IP Str
    for (cur = cur->children; cur != NULL; cur = cur->next) {
        // Is this it? Return it!
        if ((curval = xmlGetProp(cur, XML_VALU)) != NULL) {
            if (strcmp(cur->name, "IP") == 0
                && strcmp(curval, ipstr) == 0) {
                xmlFree(curval);
                return (cur);
            }
            xmlFree(curval);
            curval = NULL;
        }
    }

    // If we get here we didn't find it 
    return (NULL);
}

/**
 * qg_xml_find_child searches for a child-node matching with the
 * string 'name'. The node is returned on success, on failure
 * NULL is returned.
 */
xmlNodePtr qg_xml_find_child(xmlNodePtr parent, char *name)
{
    xmlNodePtr cur = NULL;

    // Sanity Check
    if (!parent || !name) {
        return (NULL);
    }

    // Search for child item
    for (cur = parent->children; cur != NULL; cur = cur->next) {
        // Return if found
        if (strcmp(cur->name, name) == 0) {
            return (cur);
        }
    }

    // if we get here we didn't find it
    return (NULL);
}

/**
 * qg_xml_find_prop searches for the tag-property specified by 'name' and
 * returns the XML-node if the value of the property matches the specified
 * 'value'. If nothing was found qg_xml_find_prop returne NULL. 
 */
xmlNodePtr qg_xml_find_prop(xmlNodePtr parent, char *name, char *value)
{
    xmlNodePtr cur = NULL;
    char *propdata = NULL;

    // Sanity Check
    if (!parent || !name) {
        return (NULL);
    }

    // Search all children
    for (cur = parent->children; cur != NULL; cur = cur->next) {
        // Can we get the named property?
        if ((propdata = xmlGetProp(cur, name)) != NULL) {
            // If no value is specified or the specified value matches return
            if (!value || (value && strcmp(propdata, value) == 0)) {
                xmlFree(propdata);
                return (cur);
            }
            xmlFree(propdata);
            propdata = NULL;
        }
    }

    // Default to not finding it
    return (NULL);
}

/**
 * qg_xml_print_children prints all children-tags of the XML-node
 * specified by 'parent'.
 */
void qg_xml_print_children(xmlNodePtr parent)
{
    xmlNodePtr cur = NULL;

    //no output if debug-mode is not enabled
    if (!parent || !qg_debug) {
        return;
    }

    for (cur = parent->children; cur != NULL; cur = cur->next) {
        printf("Name: %s Content: %s\n", cur->name, cur->content);
    }
}

