/*
 * $Id: quidscor.h,v 1.80 2003/10/07 06:31:31 ldemailly Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <libqg.h>
#include <signal.h>
#include <syslog.h>
#include <regex.h>

// TODO: needed ? why here ?
#ifdef QS_DRAGON
#define _XOPEN_SOURCE
#endif
#include <time.h>

// Mode defines
#define	MODE_UNDEFINED          0
#define	MODE_SNORT              1
#define MODE_SNORT_C            'S'
#define	MODE_DRAGON             2
#define MODE_DRAGON_C           'D'
#define	MODE_LOG                3
#define MODE_LOG_C              'L'

// define special for dragon
#define DRAG_SNMP_OFFSET 68
#define DR_PORT 2040

// file endings
#define RULFILEND_SNORT ".rules"
#define RULFILEND_DRAGON ".lib"

// file open mode (read, write,...)
#define FMODE_WRITE "w"
#define FMODE_READ "r"

// type of vuln-signature
#define SIG_CVE "CVE-"
#define SIG_CAN "CAN-"
#define SIG_SID "sid:"
#define SIG_LEN 4

// config checks
#define SIGN_COMM               '#'
#define SIGN_TRUE               'T'

// delimiters
#define DIRDELIM   "/"
#define CONFDELIM  " \r\n\t"

// snort defines
#define ALERTMSG_LENGTH       256
#define	SNAPLEN		     1514
#define SIDID_OFFSET            4
#define DATE_FORMAT             "%Y-%m-%d %H:%M:%S"

// dragon defines
#define ARGC_ICMP 8
#define ARGC_TCP 10
#define TCP_DEF  6
#define UDP_DEF  17
#define ICMP_DEF 1
#define DRAG_MINBUF_LEN 70
#define DRAG_LOAD_NAME "Name="
#define DRAG_LOAD_NAME_LEN 5
#define DRAG_LOAD_CVE "CVE="
#define DRAG_LOAD_CVE_LEN 4
#define DRAG_TIME_LEN 16

// Alert defines
#define	ALERT_GOOD		1
#define	ALERT_BAD		2
#define	ALERT_UNVERIFIED	3
#define ALERT_BUF_LEN        4096

// Log-files
#define AL_GOOD                 "alerts_validated.log"
#define AL_BAD                  "alerts_invalidated.log"
#define AL_UNVER                "alerts_unknown.log" 

// Directories
#define CACHE_DIR               "run/quidscor/cache/"
#define LOG_DIR                 "log/quidscor/"
#define CREATE_MOD              0700
#define MOD_PID                 0640
#define PID_FILE                "quidscor.pid"
#define MAX_PIDLEN              6
#define NUMOF_STAND_DESCR       3

// snort socket default
#define SNORT_SOCKET            "/var/log/snort/snort_alert"
// input log file
#define INPUT_LOGFNAME          "/var/log/snort/alert"

// Configfile-options
#define CO_MOD                  "MODE"
#define CO_SER                  "QGSERVER"
#define CO_LOG                  "LOGDIR"
#define CO_CAC                  "CACHEDIR"
#define CO_RUL                  "RULESDIR"
#define CO_EXP                  "EXPIRETIME"
#define CO_MAP                  "CUSTOMMAP"
#define CO_SRC                  "SOURCE"
#define CO_EXE                  "EXEC"
#define CO_IPG                  "IPGROUP"

// tokens for execution
#define TO_ADDI    "$ADDITIONAL"
#define TO_ALAR    "$ALARMMSG"
#define TO_ALER    "$ALERTSTRING"
#define TO_CLAS    "$CLASS"
#define TO_CLIE    "$CLIENTIP"
#define TO_CLPO    "$CLIENTPORT"
#define TO_CVE     "$CVE"
#define TO_EMPT    " "
#define TO_PROT    "$PROTO"
#define TO_SEPO    "$SERVERPORT"
#define TO_SERV    "$SERVERIP"
#define SHELL_EXEC "/bin/sh"
#define SHELL_EXEC_OPT "-c"

// xpath constants
#define XPATH_PROT_APP  "[@APPLICATION=\"%s\"]"
#define XPATH_PROT_SERV "[@SERVICE=\"%s\"]"

#ifdef USE_XPATH
// XPaths for reports in one report-context
#define XPATH_PROT_PORT "/REPORT/RESULT[@IP=\"%s\"]/INFOS/PORT[@PROTOCOL=\"%s\"][@VALUE=\"%d\"]"
#define XPATH_VULN_PORT "/REPORT/RESULT[@IP=\"%s\"]/VULNS/FOUND[@ID=\"%s\"][@PORT=\"%d\"]"
#define XPATH_VULN      "/REPORT/RESULT[@IP=\"%s\"]/VULNS/FOUND[@ID=\"%s\"]"
#else
// XPaths for reports in report-hash-table
#define XPATH_PROT_PORT "/RESULT[@IP=\"%s\"]/INFOS/PORT[@PROTOCOL=\"%s\"][@VALUE=\"%d\"]"
#define XPATH_VULN_PORT "/RESULT[@IP=\"%s\"]/VULNS/FOUND[@ID=\"%s\"][@PORT=\"%d\"]"
#define XPATH_VULN      "/RESULT[@IP=\"%s\"]/VULNS/FOUND[@ID=\"%s\"]"
#endif

// verification reasons
#define OUTPUT_ALERT "[A] "
#define TCP_PORT_WARNING           "TCP-PORT-WARNING"
#define VULN_ONPORT_VERIFIED       "VULN-ONPORT-VERIFIED"
#define THREAT_ONPORT_VERIFIED     "THREAT-ONPORT-VERIFIED"
#define VULN_VERIFIED              "VULN-VERIFIED"
#define THREAT_VERIFIED            "THREAT-VERIFIED"
#define VULN_TCP_PORT_VERIFIED     "TCP-PORT-VERIFIED"
#define VULN_UDP_PORT_VERIFIED     "UDP-PORT-VERIFIED"
#define VULN_TCP_SERV_VERIFIED     "TCP-SERVICE-VERIFIED"
#define VULN_UDP_SERV_VERIFIED     "UDP-SERVICE-VERIFIED"
//for now only "APPLICATION-VERIFIED" as only web servers 
//can be identified
#define VULN_TCP_APP_VERIFIED      "APPLICATION-VERIFIED"
#define VULN_UDP_APP_VERIFIED      "APPLICATION-VERIFIED"
#define VULN_NO_REPORT_FOUND       "NO-REPORT-FOUND"
#define VULN_NO_VERIFICATION       "NO-VERIFICATION"
#define VULN_NO_VULN_VERIFIED      "NO-VULN-VERIFIED"
#define VULN_NO_TCP_PORT_VERIFIED  "NO-TCP-PORT-VERIFIED"
#define VULN_NO_UDP_PORT_VERIFIED  "NO-UDP-PORT-VERIFIED"
#define VULN_NO_TCP_SERV_VERIFIED  "NO-TCP-SERVICE-VERIFIED"
#define VULN_NO_UDP_SERV_VERIFIED  "NO-UDP-SERVICE-VERIFIED"
//for now only "NO-APPLICATION-VERIFIED" as only web servers
//can be identified
#define VULN_NO_TCP_APP_VERIFIED   "NO-APPLICATION-VERIFIED"
#define VULN_NO_UDP_APP_VERIFIED   "NO-APPLICATION-VERIFIED"

// mapping strings
#define MAP_STR_ID    "ID"
#define MAP_STR_VAL   "VALUE"
#define MAP_STR_TYPE  "TYPE"
#define MAP_STR_SERV  "SERVICE"
#define MAP_STR_APP   "APPLICATION"
#define MAP_STR_CUS   "CUSTOM"
#define MAP_STR_FLAG  "FLAG"
#define MAP_STR_ESTABLISHED  "ESTABLISHED"

// default expiretime of reports
#define DEF_EXPIRE          86400  // 1 day, normal/default
#define MIN_EXPIRE           3600  // 1 hour min

// array lengths
#define TEN_ARR_LEN            10
#define SHO_ARR_LEN            20
#define TT_ARR_LEN             32
#define TFS_ARR_LEN           256

// broadcast
#define ADR_BROADCAST 0xFFFFFFFF

// strlen("CVE-0000-0000") (will break if ever there are >9999 vuln in a year)
#define CVE_STR_LENGTH 13

// constants for forking
#define FORK_SUCCESS 0
#define FORK_ERR -1

typedef enum {
    QSCE_NORMAL,       /*0 - normal exit of quidscor */
    QSCE_SOURCE,       /*1 - error opening input file */
    QSCE_ARGS,         /*2 - too few/bad arguments at command line */
    QSCE_MODE,         /*3 - not available mode selected */
    QSCE_MODE_UNDEF,   /*4 - mode not defined */
    QSCE_RULESDIR,     /*5 - no rulesdir specified */
    QSCE_LOGDIR,       /*6 - no access to logdir */
    QSCE_CACHEDIR,     /*7 - no access to cachedir */
    QSCE_LOGFILE,      /*8 - could not open logfiles */
    QSCE_USERNAME,     /*9 - no username specified */
    QSCE_QGINIT,       /*10 - QG_INIT failed */
    QSCE_NOCONF,       /*11 - no configfile specified */
    QSCE_OPENCONF,     /*12 - could not open configfile */
    QSCE_MEMORY,       /*13 - memory failure at allocation */
    QSCE_CONFILE,      /*14 - exit because of error in config-file */
    QSCE_SIG_DIR,      /*15 - error at accessing the signature directory */
    QSCE_SIG_FILE,     /*16 - error at accessing a signature file */
    QSCE_SIG_NOTFOUND, /*17 - no signatures found */
    QSCE_FORK,         /*18 - child process for daemon could not be started */
    QSCE_SOCKET,       /*19 - socket error at attempt to open one to IDS */
    QSCE_BIND,         /*20 - bind error at attempting to listen to IDS */
    QSCE_PID,          /*21 - could not open PID-file */
    QSCE_QGCVE,        /*22 - could not load supported CVEs from QG */
    QSCE_MAP,          /*23 - could not map IDs from the IDS with Qualys-IDs */
    QSCE_MAPTAB,       /*24 - an error at accessing the mapping-table */
    QSCE_REGEX,        /*25 - an error during regular expression processing */
    QSCE_QGGRPS,       /*26 - error fetching group_list */
} QC_exit;

// Alarm structure for unified processing
typedef struct _Alert {
    char *rid;
    char *alertmsg;
    char *alertclass;
    char *proto;
    u_long prottype;
    struct in_addr source_ip;
    u_short source_port;
    struct in_addr dest_ip;
    u_short dest_port;
    int qids;
    xmlNodePtr maps;
    char *quid;
    char *cve;
    char *timestr;
    char *additional;
    char *xpath;
} Alert;


// Our exec structure. Used for command execution entries
#define MAX_TOKENS 20

typedef struct _exec {
    u_int good;
    u_int bad;
    u_int unverified;
    u_int argc;
    char *argv[MAX_TOKENS];
    struct _exec *next;
} exec_t;

typedef struct _idsdb {
    xmlDocPtr siddb;          // TODO: remove me (temp data struct)
    // Main mapping IDS ids -> Infos (VA data, rules flags, service, ...)
    xmlXPathContextPtr idstoqid; 
    // Reports DB
    xmlXPathContextPtr reports;
#ifndef USE_XPATH
    xmlHashTablePtr idstoqidhash;
    xmlHashTablePtr reporthash;
#endif
    // Reverse mapping used when processing/refreshing reports for stats
    xmlXPathContextPtr qidtoids;
    // Only used for stats : TODO: remove from here / output locally ?
    u_int idsIDdbsize;
    u_int cvedbsize;
    u_int multidsIDs;
    u_int otoqidids;
    u_int otoidsqid;
    u_int estdbsize;
    u_int custom;
    u_int customoto;
} idsdb_t;

// Solaris 2.8 at least lacks u_int*_t
#ifdef __SVR4
#   define NO_U_INTX_T
#   define NO_FCNTL_O_ASYNC
#endif

#ifdef NO_U_INTX_T
typedef unsigned int u_int32_t;
typedef unsigned char u_int8_t;
typedef unsigned short u_int16_t;
#endif


/*----------------------------------------------------------------------
 * The following part has been copied from snort from 'events.h' and 
 * 'spo_alert_unixsock.h' to use the structure of the data sent over
 * the unix-socket of snort. Only 'struct pcap_pkthdr' has been replaced
 * to drop pcap.h from this headerfile and the structure Alertpkt was 
 * renamed to Snortpkt.
 *------------------------------------------------------------------- */

// Event structures from snort sources
typedef struct _Event {
    u_int32_t sig_generator;    /* which part of snort generated the alert? */
    u_int32_t sig_id;           /* sig id for this generator */
    u_int32_t sig_rev;          /* sig revision for this id */
    u_int32_t classification;   /* event classification */
    u_int32_t priority;         /* event priority */
    u_int32_t event_id;         /* event ID */
    u_int32_t event_reference;  /* reference to other events that have gone off, * such as in the case of tagged packets...  *                                 */
    struct timeval ref_time;    /* reference time for the event reference */
} Event;

/* this struct is for the alert socket code.... */
typedef struct _Snortpkt {
    u_int8_t alertmsg[ALERTMSG_LENGTH]; /* variable.. */
//paketheader - necessary for typecast of buffer
//from char * to Alertpkt -- start
    struct timeval ts;
    u_int32_t caplen;
    u_int32_t len;
//paketheader -- end
    u_int32_t dlthdr;           /* datalink header offset. (ethernet, etc.. ) */
    u_int32_t nethdr;           /* network header offset. (ip etc...) */
    u_int32_t transhdr;         /* transport header offset (tcp/udp/icmp ..) */
    u_int32_t data;
    u_int32_t val;              /* which fields are valid. (NULL could be * valids also) *                      *                                  */
    /* Packet struct --> was null */
#define NOPACKET_STRUCT 0x1
    /* no transport headers in packet */
#define NO_TRANSHDR    0x2
    u_int8_t pkt[SNAPLEN];
    Event event;
} Snortpkt;

/*-----------------------------------------------------------------------
 * End of the copied part
 *---------------------------------------------------------------------*/

typedef enum {
    UT_NOPORT,
    T_FOUND,
    T_ESTAB,
    T_NOPORT,
    U_FOUND,
    U_NOPORT
} QG_pcheck;

typedef enum {
    DET_NOMAP,
    DET_FOUND,
    DET_NOTFOUND,
    DET_SERV,
    DET_APP
} QG_detcheck;

typedef enum {
    AUTOMATIC,
    CUSTOM
} QG_mapping;

// Snort
void snort_main(char *fname);
void snort_process_alert(char *buf, size_t len);
void snort_load_alertfile(char *file, xmlDocPtr siddb);
void snort_custom(const char *buf, const char *sigbuf, char *type);

// Input log file
void log_input_main(char *inputlogfname);
void log_input_process_alert(regex_t *preg, char *line);

// Dragon
#ifdef QS_DRAGON
void dragon_main(void);
void dragon_process_alert(char *buf, size_t len);
void dragon_load_alertfile(char *file);
#endif

// CVE reference functions
void load_alertdir(char *dir, int mode);
void load_custommap(char *mapfile);
int map_quid_cve(qghandle_t *handle, char *itqfile, char *qtifile);
int load_mappings(qghandle_t *handle, char *mapfile, char *rdir, int mode, char * cachedir);
void create_map(xmlNodePtr sid_elem, xmlNodePtr q_cvelist, QG_mapping type);
xmlNodePtr search_id(xmlNodePtr idlist, char *id, char *type);
int itq_addqid(xmlNodePtr sid, xmlNodePtr qid, char *cve);
int qti_addqid(xmlNodePtr root, xmlNodePtr qid, char *sid, char *cve, int count);
int count_cve(xmlDocPtr doc);
#ifndef USE_XPATH
int create_maphash(xmlDocPtr doc);
#endif

// Alert processing functions
void process_alert(Alert *alert);
void check_alert(Alert *alert);
int findMapping (Alert *alert);
QG_pcheck check_Ports( Alert *alert);
QG_detcheck check_Detail( Alert *alert, QG_detcheck detail);
char *check_CVE( Alert *alert);
int flagValueSet (xmlNodePtr node, char* type, char* value);
void print_alert(FILE *fhandle, char *verreas, Alert *alert);
xmlNodePtr find_qids(char *str);
void alert_exec(u_int type, Alert *alert);
void free_alert(Alert *alert);

// Report caching functions
struct stat *need_update(char *cfname, u_long pexpiretime);
int refresh_host(char *addrname, char *cfname);
int find_host(u_long ip, int refreshDB);
void qualys_reports();

// Singal Handling
void sighandler(int sig);
void checkpid();
void cleanup();
int switch_to_daemon();

void quid_exit(QC_exit code);

void quid_message(QG_mtype type, const char *message,...)__attribute__((format(printf,2,3)));

// Config Parser
void parse_config(char *file);

// div. utilities
char *create_dir(char *directory);
void reform_path(char *path);
tsp_ipList ipRanges2ipList(xmlXPathObjectPtr in);
