/*
 * $Id: quidscor_cve.c,v 1.58 2003/10/09 18:29:45 ldemailly Exp $
 *
 * Copyright (c) 2003, Qualys, Inc.  All rights reserved.
 *
 * See the file LICENSE for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */
#include "quidscor.h"

idsdb_t idsdb;
int idsdb_need_init = TRUE;
extern int daemonmode;
extern int dbReadOnly;
extern int dbRefreshOnly;
extern u_long expiretime;
extern char *ipgroup;

/**
 * load_alertdir searches the directory specified by 'dir' for 
 * rule-files and calls the IDS-load_alertfile-function 
 * (snort_load_alertfile) for processing the rules.
 * PART - 2 of creation of idstoqid
 */
void load_alertdir(char *dir, int mode)
{
    char fullpath[STR_BUF_LEN];
    DIR *ourdir = NULL;
    struct dirent *file = NULL;
    xmlDocPtr siddb = NULL;
    xmlNodePtr siddbroot = NULL;

    //TODO: do not load if the cached DB is recent enough

    // Open directory
    if ((ourdir = opendir(dir)) == NULL) {
        snprintf(fullpath, STR_BUF_LEN, "Can't load sigs from \"%s\"", dir);
        perror(fullpath);
        quid_exit(QSCE_SIG_DIR);
    }

    //XML-structure for Snort-IDS with CVE
    siddb = xmlNewDoc("1.0");
    siddbroot = xmlNewDocNode(siddb, NULL, "IDS", NULL);
    xmlDocSetRootElement(siddb, siddbroot);
    // Scan dir
    while ((file = readdir(ourdir))) {
        // Rules file? Parse it
        snprintf(fullpath, STR_BUF_LEN, "%s/%s", dir, file->d_name);
        
        switch (mode) {
          case MODE_LOG:
          case MODE_SNORT:
              if (strstr(file->d_name, RULFILEND_SNORT) != NULL) {
                  snort_load_alertfile(fullpath, siddb);
              }
              break;
#ifdef QS_DRAGON
          case MODE_DRAGON:
              if (strstr(file->d_name, RULFILEND_DRAGON) != NULL) {
                  dragon_load_alertfile(fullpath);
              }
              break;
#endif
          default:
              printf("Unknown mode %d. Can't load sigs.\n", mode);
              quid_exit(QSCE_MODE);
              break;
        }
    }
    idsdb.siddb = siddb;
    idsdb.estdbsize = count_cve(idsdb.idstoqid->doc);
    quid_message(DBG, "[+] Loaded %d IDS-IDs for mapping from dir: %s\n", idsdb.idsIDdbsize, dir);

    if (idsdb.idsIDdbsize <= 0) {
        snprintf(fullpath, STR_BUF_LEN, "No sigs found in dir \"%s\"", dir);
        perror(fullpath);
        quid_exit(QSCE_SIG_NOTFOUND);
    }

    if (ourdir != NULL) {
        closedir(ourdir);
    }
}

/**
 * loads custom signature- and service-map from file
 * TODO: add error checking/handling
 * PART - 1 of creation of idstoqid
 */
void load_custommap(char *mapfile) {
    xmlDocPtr filemap = NULL;
    xmlNodePtr sigids = NULL;
    xmlXPathContextPtr mapcontext = NULL;
    xmlXPathObjectPtr signatures = NULL;

    if (mapfile != NULL) {
        quid_message(DBG, "[*] Loading custom-mapping from \"%s\"", mapfile);
        
        filemap = xmlParseFile(mapfile);
        mapcontext = xmlXPathNewContext(filemap);

        // get ID-mappings
        signatures = xmlXPathEvalExpression("/MAP", mapcontext);

        if (!xmlXPathNodeSetIsEmpty(signatures->nodesetval)) {
            sigids = signatures->nodesetval->nodeTab[0]->children;
            create_map(sigids, NULL, CUSTOM);
        }

        xmlXPathFreeObject(signatures);
        xmlXPathFreeContext(mapcontext);
        xmlFreeDoc(filemap);
    }

    return;
}


/**
 * PART - 3 of creation of idstoqid
 */
int map_quid_cve(qghandle_t *handle, char *itqfile, char *qtifile) 
{
    xmlNodePtr q_cvelist = NULL;
    xmlNodePtr sid_elem = NULL;
    qgresponse_t *cveList;

    // Load from QG+file and recompute:
    cveList = qg_cve_list(handle);
    if (cveList == NULL) {
        quid_message(ERR, strqg_error("*** Supported CVE loading failed"));
        quid_exit(QSCE_QGCVE);
    }
    if ((q_cvelist = qg_xmlroot(cveList->xml, "VULNMAP")) == NULL) {
        qg_free_response(cveList);
        return FALSE;
    }

    sid_elem = xmlDocGetRootElement(idsdb.siddb);
    sid_elem = sid_elem->children;

    create_map(sid_elem, q_cvelist, AUTOMATIC);
    
    xmlFreeDoc(idsdb.siddb);
    idsdb.siddb = NULL;

    xmlSaveFile(itqfile, idsdb.idstoqid->doc);
    xmlSaveFile(qtifile, idsdb.qidtoids->doc);
    
    if (!daemonmode) {
        quid_message(DBG, "Statistical data:");
        quid_message(DBG, "[+] Loaded %d CVEs for mapping with QualysGuard", idsdb.cvedbsize);
        quid_message(DBG, "[+] IDS-IDs with more than one CVE: %d", idsdb.multidsIDs);
        quid_message(DBG, "[+] %d IDS-IDs with \"established\"-flag found", idsdb.estdbsize);
        quid_message(DBG, "Custom mappings:");
        quid_message(DBG, "[+] %d custom mappings", idsdb.custom);
        quid_message(DBG, "[+] %d 1to1 QualysID to IDS-ID mappings", idsdb.customoto);
        quid_message(DBG, "Automatic mappings:");
        quid_message(DBG, "[+] %d 1to1 IDS-ID to QualysID mappings", idsdb.otoidsqid);
        quid_message(DBG, "[+] %d 1to1 QualysID to IDS-ID mappings", idsdb.otoqidids);
        quid_message(DBG, "Overall used IDs:");
        quid_message(DBG, "[+] %d unique mapped IDS-IDs\n\t(including IDS-ID to service and application mappings)", count_cve(idsdb.idstoqid->doc));
        quid_message(DBG, "[+] %d unique mapped QualysIDs", count_cve(idsdb.qidtoids->doc));
    }
    qg_free_response(cveList);
    return TRUE;
}

void qs_freeHashEntry(void *data, xmlChar *name) {
    xmlXPathContextPtr realData = (xmlXPathContextPtr) data;
    if (realData != NULL) {
        if (realData->doc != NULL) {
            xmlFreeDoc(realData->doc);
        }
        xmlXPathFreeContext(realData);
    }
}

/**
 * load_mappings loads either mapping-structures from files or creates the
 * mapping-structures from the custom-mappings, the IDS-rules and 
 * CVE to Qualys-ID mappings from QualysGuard
 */
int load_mappings(qghandle_t *handle, char *mapfile, char *rdir, int mode, char * cachedir) {
    char *itqfile = strdup(cachedir);
    char *qtifile = strdup(cachedir);
    xmlDocPtr idstoqid = NULL;
    xmlDocPtr qidtoids = NULL;
    xmlNodePtr itqStart = NULL;
    xmlNodePtr qtiStart = NULL;
    int fresh = FALSE;
    int mapsuccess = 0;

    itqfile = qg_strappend(itqfile, "/itqmap.xml");
    qtifile = qg_strappend(qtifile, "/qtimap.xml");

    // DB init
    if (idsdb_need_init) {
        // first time, clear the structure
        idsdb_need_init = FALSE;
    } else {
        // 2nd and up time: free up already allocated
        // structure (if needed)
#ifndef USE_XPATH
        if (idsdb.idstoqidhash != NULL) {
            // no deallocator here because what is stored
            // is pointer into the idstoqid xml doc which is
            // free'ed below
            xmlHashFree(idsdb.idstoqidhash, NULL);
        }
        if (idsdb.reporthash != NULL) {
            // on the other hand this hash table contains
            // new context objects and xml docs which need
            // to be freed
            xmlHashFree(idsdb.reporthash, qs_freeHashEntry);
        }
#endif
        if (idsdb.siddb != NULL) {
            xmlFreeDoc(idsdb.siddb);
        }
        if (idsdb.idstoqid != NULL) {
            if (idsdb.idstoqid->doc != NULL) {
                xmlFreeDoc(idsdb.idstoqid->doc);
            }
            xmlXPathFreeContext(idsdb.idstoqid);
        }
        if (idsdb.reports != NULL) {
            if (idsdb.reports->doc != NULL) {
                xmlFreeDoc(idsdb.reports->doc);
            }
            xmlXPathFreeContext(idsdb.reports);
        }
        if (idsdb.qidtoids != NULL) {
            if (idsdb.qidtoids->doc != NULL) {
                xmlFreeDoc(idsdb.qidtoids->doc);
            }
            xmlXPathFreeContext(idsdb.qidtoids);
        }
        // TODO: other global variables to free ?
    }
    // Reset or initialize
    memset(&idsdb, 0, sizeof(idsdb));

    // Main file tested for cache freshness
    fresh = (need_update(itqfile, expiretime) != NULL);
    if (dbReadOnly || fresh) {
        if (fresh) {
            quid_message(DBG, "Mapping DB (%s) is recent enough, no refresh", 
                         itqfile);
        }
        // Load from file:
        idstoqid = xmlParseFile(itqfile);
        if (idstoqid == NULL) {
            if (fresh) {
                quid_message(ERR, "Corrupt DB file (%s)", itqfile);
            } else {
                quid_message(ERR, "No valid DB file (%s) present.\n    Run with -0 at least once.", itqfile);
            }
            exit(QSCE_MAP);
        }
        qidtoids = xmlParseFile(qtifile);
        idsdb.idstoqid = xmlXPathNewContext(idstoqid);
        idsdb.qidtoids = xmlXPathNewContext(qidtoids);
#ifndef USE_XPATH
        create_maphash(idsdb.idstoqid->doc);
#endif
        qualys_reports();

        // TODO: save/load or recompute the stats - double check if they are used
        free(itqfile);
        free(qtifile);
        return TRUE;
    }

    quid_message(DBG, "Outdated Mapping DB (%s), refreshing all", itqfile);

    // create XML-Documents for the mappings
    idstoqid = xmlNewDoc("1.0");
    itqStart = xmlNewDocNode(idstoqid, NULL, "IDSTOQID", NULL);
    xmlDocSetRootElement(idstoqid, itqStart);

    qidtoids = xmlNewDoc("1.0");
    qtiStart = xmlNewDocNode(qidtoids, NULL, "QIDTOIDS", NULL);
    xmlDocSetRootElement(qidtoids, qtiStart);

    idsdb.idstoqid = xmlXPathNewContext(idstoqid);
    idsdb.qidtoids = xmlXPathNewContext(qidtoids);

    // load rules and custom mapping
    load_custommap(mapfile); 
    load_alertdir(rdir, mode);
    mapsuccess = map_quid_cve(handle, itqfile, qtifile);
#ifndef USE_XPATH
    create_maphash(idsdb.idstoqid->doc);
#endif

 // Pre Fetch reports that are outdated:
    if (!dbReadOnly) {
        tsp_ipList ipList, ipNode;

        quid_message(DBG, "[*] Fetching data for IPs from group \"%s\"", ipgroup); 
        ipList = qg_ips_from_group(handle, ipgroup);
        if (ipList == NULL) {
            quid_message(ERR, "%s group=\"%s\"", strqg_error("ip for group failure"), 
                         ipgroup);
            quid_exit(QSCE_QGGRPS);
        }
        // there should be a quid_message(INFO,...)
        printf("[*] VA database for %ld ips\n", ipList->count);

        for (ipNode = ipList; ipNode != NULL; ipNode = ipNode->next) {
            struct in_addr ip;
            char *target;
            ip.s_addr = ipNode->ip;
            target = inet_ntoa(ip);
            quid_message(DBG, "\t%s", target);
            find_host(ip.s_addr, FALSE); // refresh once at the end
        }

        freeIpList(ipList);
        ipList = NULL;
    }
    // Initial loading of currently known reports (always):

    qualys_reports();

    free(itqfile);
    free(qtifile);
    return mapsuccess;
}

/**
 * search_id searches a level of xml-nodes for a matching property
 */
xmlNodePtr search_id(xmlNodePtr idlist, char *id, char *type)
{
    char *runid = NULL;
    xmlNodePtr runlist = idlist;

    while (runlist != NULL) {
        // what kind of mapping-element should be checked
        runid = xmlGetProp(runlist, type);
        
        if (runid == NULL) {
            runlist = runlist->next;
            continue;
        }

        if (!strcmp(id, runid)) {
            if (runid) {
                xmlFree(runid);
            }
            return runlist;
        }

        if (runid) {
            xmlFree(runid);
            runid = NULL;
        }

        runlist = runlist->next;
    }

    if (runid) {
        xmlFree(runid);
    }

    return runlist;
}

/**
 * create_map creates a xml-mappingstructure
 */
void create_map(xmlNodePtr sid_elem, xmlNodePtr q_cvelist, QG_mapping type)
{
    int otoqidids = 0;
    int otoidsqid = 0;
    xmlNodePtr itqStart = NULL;
    xmlNodePtr qtiStart = NULL;

    itqStart = xmlDocGetRootElement(idsdb.idstoqid->doc);
    qtiStart = xmlDocGetRootElement(idsdb.qidtoids->doc);
    while (sid_elem != NULL) {
        char *sid = NULL;
        char *intype = NULL;
        xmlNodePtr cve_elem = NULL;
        xmlNodePtr idsInNode = NULL;
        int mappedqids = 0;
        
        // check for type of mapping (ID, service, application)
        if ((intype = xmlGetProp(sid_elem, MAP_STR_TYPE)) == NULL) {
            sid_elem = sid_elem->next;
            continue;
        }

        // get value of mapping-type
        if ((sid = xmlGetProp(sid_elem, MAP_STR_VAL)) == NULL) {
            xmlFree(intype);
            sid_elem = sid_elem->next;
            continue;
        }

        cve_elem = sid_elem->children;
        while (cve_elem != NULL) { 
            char *cve = NULL;
            char *mapping = NULL;
            char *outtype = NULL;
            xmlNodePtr cvefound = NULL;
            xmlNodePtr qid = NULL;

            switch(type) {
                case AUTOMATIC:
                    if ((outtype = xmlGetProp(cve_elem, MAP_STR_TYPE)) == NULL) {
                        cve_elem = cve_elem->next;
                        continue;
                    }

                    if ((cve = xmlGetProp(cve_elem, MAP_STR_VAL)) == NULL) {
                        xmlFree(outtype);
                        cve_elem = cve_elem->next;
                        continue;
                    }

                    mapping = cve + SIG_LEN;
                    cvefound = search_id(q_cvelist->children, mapping, MAP_STR_ID);
                    if (cvefound == NULL) {
                        cve_elem = cve_elem->next;
                        xmlFree(cve);
                        xmlFree(outtype);
                        continue;
                    }
                    xmlFree(outtype);
                    outtype = NULL;

                    qid = cvefound->children;
                    break;
                case CUSTOM:
                    if ((outtype = xmlGetProp(cve_elem, MAP_STR_TYPE)) == NULL) {
                        cve_elem = cve_elem->next;
                        continue;
                    }

                    if ((cve = xmlGetProp(cve_elem, MAP_STR_VAL)) == NULL) {
                        xmlFree(outtype);
                        cve_elem = cve_elem->next;
                        continue;
                    }
                    xmlFree(outtype);
                    outtype = NULL;
                    qid = cve_elem;
                    mapping = MAP_STR_CUS;
                    break;
                default:
                    cve_elem = cve_elem->next;
                    continue;
            }

            idsInNode = search_id(itqStart->children, sid, MAP_STR_VAL);
            if (idsInNode == NULL) {
                idsInNode = xmlNewChild(itqStart, NULL, "IN", NULL);
                xmlNewProp(idsInNode, MAP_STR_TYPE, intype);
                xmlNewProp(idsInNode, MAP_STR_VAL, sid);
            }
            
            mappedqids += itq_addqid(idsInNode, qid, mapping);

            otoqidids = qti_addqid(qg_xmlroot(idsdb.qidtoids->doc, "QIDTOIDS"), 
                                   qid, sid, mapping, otoqidids);

            cve_elem = cve_elem->next;
            xmlFree(cve);
            // Next iteration in the inner loop, cve reset to NULL
        }

        if (mappedqids == 1) {
            otoidsqid++;
        }

        sid_elem = sid_elem->next;
        xmlFree(intype); 
        xmlFree(sid);
        // Next iteration in the outter loop, sid and intype reset to NULL
    }

    if (type == CUSTOM) {
        idsdb.custom = otoidsqid;
        idsdb.customoto = otoqidids;
    } else {
        idsdb.otoidsqid = otoidsqid;
        idsdb.otoqidids = otoqidids;
    }
}

/**
 * itq_addqid adds a childnode to an IDS-ID 
 */
int itq_addqid(xmlNodePtr sid, xmlNodePtr qid, char *cve)
{
    xmlNodePtr runqid = qid;
    int added = 0;

    while (runqid != NULL) {
        char *qidstr = NULL;
        char *outtype = NULL;
        char *proptype = MAP_STR_ID;
        xmlNodePtr foundqid = NULL;
        xmlNodePtr foundcve = NULL;

        if ((qidstr = xmlGetProp(runqid, MAP_STR_ID)) == NULL) {
            if ((outtype = xmlGetProp(runqid, MAP_STR_TYPE)) == NULL) {
                runqid = runqid->next;
                continue;
            }
        
            if ((qidstr = xmlGetProp(runqid, MAP_STR_VAL)) == NULL) {
                xmlFree(outtype);
                runqid = runqid->next;
                continue;
            }

            proptype = MAP_STR_VAL;
        }
                
        if ((foundqid = search_id(sid->children, qidstr, proptype)) == NULL) {
            added++;
            foundqid = xmlNewChild(sid, NULL, "OUT", NULL);
            xmlNewProp(foundqid, MAP_STR_TYPE, outtype ? outtype : MAP_STR_ID);
            xmlNewProp(foundqid, MAP_STR_VAL, qidstr);
            
            foundcve = xmlNewChild(foundqid, NULL, "COR", NULL);
            xmlNewProp(foundcve, MAP_STR_ID, cve);
        } else {
            if ((foundcve = search_id(foundqid->children, cve, MAP_STR_ID)) == NULL) {
                foundcve = xmlNewChild(foundqid, NULL, "COR", NULL);
                xmlNewProp(foundcve, MAP_STR_ID, cve);
            }
        }

        if (qidstr) {
            xmlFree(qidstr);
            qidstr = NULL;
        }
        runqid = runqid->next;
        if (outtype) {
            xmlFree(outtype);
            outtype = NULL;
        }
    }
    
    return added;
}

/**
 * qti_addqid adds a Qualys-ID, IDS-ID and the mapping-cause (CVE, custom) to
 * the mapping-tabel from Qualys-ID to IDS-ID and returns the number of 
 * one-to-one-mappings
 */
int qti_addqid(xmlNodePtr root, xmlNodePtr qid, char *sid, char *cve, int count)
{
    xmlNodePtr runqid = qid;
    int oto = count;

    while (runqid != NULL) {
        char *qidstr = NULL;
        char *outtype = NULL;
        xmlNodePtr qidnode = NULL;
        xmlNodePtr idsnode = NULL;
        xmlNodePtr cvenode = NULL;

        if ((qidstr = xmlGetProp(runqid, MAP_STR_ID)) == NULL) {
            if ((outtype = xmlGetProp(runqid, MAP_STR_TYPE)) == NULL) {
                runqid = runqid->next;
                continue;
            }
            if (strncmp(outtype, MAP_STR_ID, sizeof(MAP_STR_ID))) {
                xmlFree(outtype);
                runqid = runqid->next;
                continue;
            }
        
            if ((qidstr = xmlGetProp(runqid, MAP_STR_VAL)) == NULL) {
                xmlFree(outtype);
                runqid = runqid->next;
                continue;
            }
        }
                
        if ((qidnode = search_id(root->children, qidstr, MAP_STR_VAL)) == NULL) {
            oto++;
            qidnode = xmlNewChild(root, NULL, "IN", NULL);
            xmlNewProp(qidnode, MAP_STR_ID, qidstr);
            xmlNewProp(qidnode, "MAP", "1");
            
            idsnode = xmlNewChild(qidnode, NULL, "OUT", NULL);
            xmlNewProp(idsnode, MAP_STR_ID, sid);

            cvenode = xmlNewChild(idsnode, NULL, "COR", NULL);
            xmlNewProp(cvenode, MAP_STR_ID, cve);
        } else {
            if ((idsnode = search_id(qidnode->children, sid, MAP_STR_VAL)) == NULL) {
                if (xmlUnsetProp(qidnode, "MAP") == 0) {
                    oto--;
                }

                idsnode = xmlNewChild(qidnode, NULL, "OUT", NULL);
                xmlNewProp(idsnode, MAP_STR_ID, sid);

                cvenode = xmlNewChild(idsnode, NULL, "COR", NULL);
                xmlNewProp(cvenode, MAP_STR_ID, cve);
            } else {
                if ((cvenode = search_id(idsnode->children, cve, MAP_STR_ID)) == NULL) {
                    cvenode = xmlNewChild(idsnode, NULL, "COR", NULL);
                    xmlNewProp(cvenode, MAP_STR_ID, cve);
                }
            }
        }
        
        if (qidstr) {
            xmlFree(qidstr);
            qidstr = NULL;
        }

        if (outtype) {
            xmlFree(outtype);
            outtype = NULL;
        }

        runqid = runqid->next;
    }
    return oto;
}

/**
 * count_cve returns the number of primary-nodes in a xml-document.
 */
int count_cve(xmlDocPtr doc)
{
    int count = 0;
    xmlNodePtr cur = NULL;

    if ((cur = xmlDocGetRootElement(doc)) == NULL) {
        return 0;
    }

    for (cur = cur->children; cur != NULL; cur = cur->next) {
        if (cur->type == XML_ELEMENT_NODE) {
            count++;
        }
    }

    return count;
}

#ifndef USE_XPATH
int create_maphash(xmlDocPtr doc) {
    int count = 0;
    xmlNodePtr cur = NULL;
    
    count = count_cve(doc);

   if (!count) {
        return FALSE;
    }

    idsdb.idstoqidhash = xmlHashCreate(count);
    
    if ((cur = xmlDocGetRootElement(doc)) == NULL) {
        return FALSE;
    }
    
    for (cur = cur->children; cur != NULL; cur = cur->next) {
        if (cur->type == XML_ELEMENT_NODE) {
            char *id = NULL;
            id = xmlGetProp(cur, MAP_STR_VAL);
            
            xmlHashAddEntry(idsdb.idstoqidhash, id, cur->children);
            xmlFree(id);
        }
    }
    return TRUE;
}
#endif
